#include <QtGui>
#include "zdownloader.h"

ZDownloader::ZDownloader(QWidget *parent, Qt::WindowFlags flags, bool standalone)
	: QWidget(parent, flags)
{
  #ifdef Q_WS_X11
    p = new QProcess(this);
    p->start("uname -m"); 
    p->waitForFinished(-1);
    QString archOS =  p->readAllStandardOutput();
    archOS.remove('\n');
    _currentOS = "linux" + archOS.right(2);
    _currentOSsuffix = "";
    qDebug() << "Linux version " + archOS;
  #endif
  #ifdef Q_WS_WIN
    qDebug() << "Windows version";
    qDebug() << QSysInfo::windowsVersion();
    qDebug() << QSysInfo::WindowsVersion;
    _currentOS = "win32";
    _currentOSsuffix = ".exe";
  #endif
  #ifdef Q_WS_MAC
    qDebug() << "Mac version";
    _currentOS = "macos";
    _currentOSsuffix = ".bin";
  #endif
  if(_currentOS == "") {
    qDebug() << "We don't support that version yet...";
    return;
  }

  _appLocation = QFileInfo(QApplication::arguments().at(0)).canonicalPath();
  if(!_appLocation.isEmpty())
    _appLocation += "/";
  preferredLanguage = QLocale::system().name().split('_')[0];
  QApplication::installTranslator(&translator);
  if(preferredLanguage != "cs")
    preferredLanguage = "en";
  translator.load(":/zdownloader_" + preferredLanguage + ".gm");

  _target = ToMa;
  _appName = "ZDownloader";
  _appVersion = "1.1.3";
  downloadVersion = "version";
  downloadCurrent = "";
  avgDownloadSpeed = 0;
  downloadBytesReceived = 0;
  downloadBytesTotal = 0;

  lblInfo = new QLabel(targetAppInfo, this);
  lblSpeed = new QLabel(this);
  lblData = new QLabel(this);
  pbrDownload = new QProgressBar(this);
  btnDownload = new QPushButton("&Download", this);
  btnDownload->setMaximumWidth(100);
  btnCancelDownload = new QPushButton("Cancel");
	btnCancelDownload->setEnabled(false);
  btnExit= new QPushButton("&Exit");
	connect(btnDownload, SIGNAL(clicked()), this, SLOT(downloadPressed()));

  QLabel *lblLogo = new QLabel("");  
  lblLogo->setPixmap(QPixmap(":/images/myappR.png"));  
  lblLogo->adjustSize(); 

  lblDownloadedName = new QLabel("");  

  QHBoxLayout *hblInfo = new QHBoxLayout;
  hblInfo->addWidget(lblLogo);
  hblInfo->addSpacing(10);
  hblInfo->addWidget(lblInfo);

  QVBoxLayout *vblProgressDownload = new QVBoxLayout;
  vblProgressDownload->addWidget(pbrDownload);
  vblProgressDownload->addWidget(lblData);
  vblProgressDownload->addWidget(lblSpeed);

  QHBoxLayout *hblButtons = new QHBoxLayout;
  hblButtons->addWidget(btnDownload, 0, Qt::AlignRight);

  QVBoxLayout *vblMain = new QVBoxLayout;
  vblMain->addLayout(hblInfo);
  vblMain->addSpacing(10);
  vblMain->addWidget(lblDownloadedName);
  vblMain->addLayout(vblProgressDownload);
  vblMain->addLayout(hblButtons);

  setLayout(vblMain);

  QIcon icon = QIcon(":/images/myappR.png");
  setWindowIcon(icon);

  tmrDownloadInfo = new QTimer(this);
  connect(tmrDownloadInfo, SIGNAL(timeout()), this, SLOT(showDownloadInfo()));
  
  move((QApplication::desktop()->width() - width())/2,
       (QApplication::desktop()->height() - height())/2);

  btnDownload->setText(tr("&Check version"));

  setTarget(_target);

  if(standalone)
    downloadVersions();
}

ZDownloader::~ZDownloader()
{
}

void ZDownloader::downloadPressed()
{
  if(versions.count()) { 
    downloadingVersionFlag = false;
    download(downloadCurrent);
  } else    
    downloadVersions();
}

void ZDownloader::downloadVersions()
{
  versions.clear(); 
  downloadingVersionFlag = true;
  download(downloadVersion);
}

void ZDownloader::downloadOnce(QString source)
{
  downloadingVersionFlag = false;
  downloadFile = source;
  QString ver;
  if((ver = version((downloadFile == _appName ? false : true))) != NULL)
    download(downloadFile + "_" + _currentOS + "_" + ver + _currentOSsuffix);
}

void ZDownloader::download(QString source)
{
	QUrl url(downloadPath + source);
  lblDownloadedName->setText(source);
 
  if(!downloadingVersionFlag) {
	  QString filename = _appLocation + QFileInfo(url.path()).fileName();
    if(QFile::exists(filename))
    {
      QFile::remove(filename);
    }
   
    file = new QFile(filename);
    if(!file->open(QIODevice::WriteOnly))
    {
      QMessageBox::critical(this, targetAppName + " ZDownloader", 
        tr("Unable to save the file %1: %2.")
        .arg(filename).arg(file->errorString()));
      delete file;
      file = NULL;
      return;
    }
  }

  m_dtStartTime = QDateTime::currentDateTime();
	downloadRequestAborted = false;
	reply = manager.get(QNetworkRequest(url));
	connect(reply, SIGNAL(finished()), this, SLOT(downloadFinished()));
	connect(reply, SIGNAL(readyRead()), this, SLOT(downloadReadyRead()));
  if(!downloadingVersionFlag)
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(downloadProgress(qint64,qint64)));
 
  if(!downloadingVersionFlag) {
    tmrDownloadInfo->start(750);
    btnDownload->setEnabled(false);
    btnCancelDownload->setEnabled(true);
  }
}

void ZDownloader::downloadProgress(qint64 bytesReceived, qint64 bytesTotal)
{
	if(downloadRequestAborted)
		return;
	pbrDownload->setValue(bytesReceived);
  if(bytesTotal <= 0) {
    QList<QByteArray> etag = reply->rawHeader("ETag").split('-'); 
    qint64 remoteFileSize = etag[1].toLongLong(0, 16);
    if(remoteFileSize > 0) {
	    pbrDownload->setMaximum(remoteFileSize);
      downloadBytesTotal = remoteFileSize;
    }
  } else {
	  pbrDownload->setMaximum(bytesTotal);
    downloadBytesTotal = bytesTotal;
  }

  qint64 lTotalMsecondsElapsed = QDateTime::currentDateTime().toMSecsSinceEpoch() - m_dtStartTime.toMSecsSinceEpoch();
  avgDownloadSpeed = (bytesReceived / lTotalMsecondsElapsed) * 1000;    
  downloadBytesReceived = bytesReceived;
}

void ZDownloader::showDownloadInfo()
{
  lblSpeed->setText("speed: " + dataSumarize(avgDownloadSpeed, true));  
  lblData->setText("size: " + dataSumarize(downloadBytesReceived, false) + " / " + dataSumarize(downloadBytesTotal, false));  
}

void ZDownloader::downloadReadyRead()
{
  if(!downloadingVersionFlag) {
    if(file)
      file->write(reply->readAll());
  } else {
    versions += reply->readAll();
  }
}

void ZDownloader::cancelDownload()
{
	downloadRequestAborted = true;
	reply->abort();
	btnDownload->setEnabled(true);
	btnCancelDownload->setEnabled(false);
  tmrDownloadInfo->stop();
  
  if(file) {
    file->close();
    file->remove();
    delete file;
    file = NULL;
  }
}

void ZDownloader::downloadFinished()
{
  if(!downloadingVersionFlag) {
    if(downloadRequestAborted) {
      if(file) {
        file->close();
        file->remove();
        delete file;
        file = NULL;
      }
      reply->deleteLater();
      btnDownload->setEnabled(true);
      btnCancelDownload->setEnabled(false);
      return;
    }
  }
 
	downloadReadyRead();
	//pbrDownload->setVisible(false);
	btnDownload->setEnabled(true);
	btnCancelDownload->setEnabled(false);

  if(!downloadingVersionFlag) {
    file->flush();
    file->setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner | QFile::ReadGroup | QFile::ExeGroup | QFile::ReadOther | QFile::ExeOther);
    file->close();

    QString executableFileName = _appLocation + downloadFile + _currentOSsuffix;
    if(!file->rename(executableFileName)) {
      QFile::remove(executableFileName);
      file->rename(executableFileName);
    }

    if(QProcess::startDetached(QFileInfo(executableFileName).absoluteFilePath()))
      qApp->quit();
  }

	if(reply->error())
	{
		QMessageBox::critical(this, tr("Download failed"), tr("Failed: %1").arg(reply->errorString()));
	}
 
	reply->deleteLater();
	reply = NULL;
	file = NULL;

  tmrDownloadInfo->stop();

  if(downloadingVersionFlag) {

    QString ver;
    if((ver = version(downloadFile == _appName ? false : true)) != NULL) {
      downloadCurrent = downloadFile + "_" + _currentOS + "_" + ver + _currentOSsuffix;
      emit gotVersion(ver);
    }
    btnDownload->setText(tr("&Download"));
  }
}

QString ZDownloader::version(bool baseApp)
{
  if(versions.count()) {
    QStringList lines = QString(versions).split('\n');
    foreach(QString line, lines)
    {
      if(line.contains(_currentOS)) {
        QStringList fields = line.split('-');
        if(baseApp)
          return fields[1];
        else
          return fields[2];
      }
    }
  }
  return NULL;
}

QString ZDownloader::appName()
{
  return _appName;
}

QString ZDownloader::appVersion()
{
  return _appVersion;
}

QString ZDownloader::osVer()
{
  return _currentOS;
}

QString ZDownloader::osSuffix()
{
  return _currentOSsuffix;
}

QString ZDownloader::self()
{
  return appName() + "_" + osVer() + "_" + version(false) + osSuffix();
}

void ZDownloader::setTarget(int target)
{
  _target = target;

  switch(target) {
    case 1:
      targetAppName = "ToMa";
      targetAppNamePath = "projects/toma";
      targetAppInfo = tr("This is downloader for <b>Tournament Manager</b>.<br />Software created by <b>Zakrava</b> for <b>FIM UHK</b>.<br />Copyright &copy; 2014-2015 Zakrava. All rights reserved.");
      targetAppTitle = tr("Tournament Manager");
      break;
    default:
      targetAppName = "ZDownloader";
      targetAppNamePath = "projects/zdownloader";
      targetAppInfo = tr("This is implemented Zakrava Downloader itself as standalone application");
      targetAppTitle = tr("ZDownloader");
      break;
  }

  downloadFile = targetAppName; //"ToMa";
  downloadPath = "http://zakrava.eu/" + targetAppNamePath + "/";
  lblInfo->setText(targetAppInfo);
  setWindowTitle(targetAppTitle + tr(" - ZDownloader"));
}

QString ZDownloader::dataSumarize(double size, bool speed)
{
  QString result = "";
  QStringList units;
  units << "B" << "kB" << "MB" << "GB" << "TB" << "PB" << "EB";
  int unit = 0;
  
  while(size > 1024) {
    size /= 1024;
    ++unit;
  }

  result = result.sprintf("%0.2f", size) + " " + units[unit];
  
  if(speed)
    result += "/s";
  
  return result;
}
